@@ -2,10 +2,11 @@ package Devel::CheckCompiler;
 use strict;
 use warnings;
 use 5.008001;
-our $VERSION = '0.04';
+our $VERSION = '0.06';
 use parent qw/Exporter/;
 
 our @EXPORT = qw/check_c99 check_c99_or_exit check_compile/;
+use Config;
 use ExtUtils::CBuilder;
 
 my $C99_SOURCE = <<'C99';
@@ -16,6 +17,9 @@ int test_c99() {
     int i = 0;
     i++;
     int j = i - 1; // another C99 feature: declaration after statement
+    // another C99 feature: for loop variable declarations
+    for (int k = 0; k < 3; i++) {
+    }
     return j;
 }
 C99
@@ -31,6 +35,28 @@ sub check_c99_or_exit {
     };
 }
 
+sub _is_gcc {
+    return 0 if $Config{gccversion} eq '';
+    # For clang on MacOSX and *BSD distributions
+    return 0 if $Config{gccversion} =~ m/clang/i;
+    # For Intel C and C++ compiler
+    return 0 if $Config{gccversion} =~ m/intel/i;
+
+    return 1;
+}
+
+sub _gcc_version {
+    if ($Config{gccversion} =~ m/^(\d+)\.(\d+)\.(\d+)/) {
+        return {
+            major => $1,
+            minor => $2,
+            patch => $3,
+        };
+    }
+
+    return;
+}
+
 sub check_compile {
     my ($src, %opt) = @_;
 
@@ -43,14 +69,24 @@ sub check_compile {
     $tmpfile->print($src);
     $tmpfile->close();
 
+    my $compile_flags;
+    if (_is_gcc()) {
+        my $gcc_version = _gcc_version();
+        # '-std=gnu11' is default from GCC 5
+        if (defined $gcc_version && $gcc_version->{major} < 5) {
+            $compile_flags = ['-std=c99'];
+        }
+    }
+
     my $objname = eval {
-        $cbuilder->compile(source => $tmpfile->filename);
+        $cbuilder->compile(source => $tmpfile->filename, extra_compiler_flags => $compile_flags);
     };
     if ($objname) {
         my $ret = 1;
         if ($opt{executable}) {
             my $execfile = eval {
-                $cbuilder->link_executable(objects => $objname);
+                $cbuilder->link_executable(objects => $objname,
+                                           extra_linker_flags => $opt{extra_linker_flags});
             };
 
             if ($execfile) {
@@ -109,6 +145,11 @@ Possible options are:
 
 Check to see if generating executable is possible if this parameter is true.
 
+=item extra_linker_flags : Str | ArrayRef[Str]
+
+Any additional flags you wish to pass to the linker. This option is used
+only when C<executable> option specified.
+
 =back
 
 =back
